@@ -30,6 +30,11 @@
 #endif
 #define LRU11_DO_NOT_DEFINE_OUT_OF_CLASS_METHODS
 
+#if defined(HAVE_LIBDL) && !defined(_GNU_SOURCE)
+// Required for dladdr() on Cygwin
+#define _GNU_SOURCE
+#endif
+
 #include <errno.h>
 #include <stdlib.h>
 
@@ -48,6 +53,12 @@
 
 #include "proj_config.h"
 
+#if defined(WINAPI_FAMILY) && (WINAPI_FAMILY == WINAPI_FAMILY_APP)
+#define UWP 1
+#else
+#define UWP 0
+#endif
+
 #ifdef _WIN32
 #include <shlobj.h>
 #include <windows.h>
@@ -692,10 +703,21 @@ std::unique_ptr<File> FileWin32::open(PJ_CONTEXT *ctx, const char *filename,
                                      ? FILE_ATTRIBUTE_READONLY
                                      : FILE_ATTRIBUTE_NORMAL;
     try {
+#if UWP
+        CREATEFILE2_EXTENDED_PARAMETERS extendedParameters;
+        ZeroMemory(&extendedParameters, sizeof(extendedParameters));
+        extendedParameters.dwSize = sizeof(extendedParameters);
+        extendedParameters.dwFileAttributes = dwFlagsAndAttributes;
+        HANDLE hFile = CreateFile2(
+            UTF8ToWString(std::string(filename)).c_str(), dwDesiredAccess,
+            FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+            dwCreationDisposition, &extendedParameters);
+#else  // UWP
         HANDLE hFile = CreateFileW(
             UTF8ToWString(std::string(filename)).c_str(), dwDesiredAccess,
             FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, nullptr,
             dwCreationDisposition, dwFlagsAndAttributes, nullptr);
+#endif // UWP
         return std::unique_ptr<File>(hFile != INVALID_HANDLE_VALUE
                                          ? new FileWin32(filename, ctx, hFile)
                                          : nullptr);
@@ -1110,7 +1132,7 @@ void proj_context_set_sqlite3_vfs_name(PJ_CONTEXT *ctx, const char *name) {
  * already.
  * @return The path to the PROJ user writable directory.
  * @since 7.1
-*/
+ */
 
 const char *proj_context_get_user_writable_directory(PJ_CONTEXT *ctx,
                                                      int create) {
@@ -1136,6 +1158,9 @@ const char *proj_context_get_user_writable_directory(PJ_CONTEXT *ctx,
             wPath.resize(wcslen(wPath.data()));
             path = NS_PROJ::WStringToUTF8(wPath);
 #else
+#if UWP
+        if (false) {
+#else  // UWP
         wchar_t *wPath;
         if (SHGetKnownFolderPath(FOLDERID_LocalAppData, 0, nullptr, &wPath) ==
             S_OK) {
@@ -1143,6 +1168,7 @@ const char *proj_context_get_user_writable_directory(PJ_CONTEXT *ctx,
             std::string str = NS_PROJ::WStringToUTF8(ws);
             path = str;
             CoTaskMemFree(wPath);
+#endif // UWP
 #endif
         } else {
             const char *local_app_data = getenv("LOCALAPPDATA");
@@ -1181,12 +1207,12 @@ const char *proj_context_get_user_writable_directory(PJ_CONTEXT *ctx,
 }
 
 /** Get the URL endpoint to query for remote grids.
-*
-* @param ctx PROJ context, or NULL
-* @return Endpoint URL. The returned pointer would be invalidated
-* by a later call to proj_context_set_url_endpoint()
-* @since 7.1
-*/
+ *
+ * @param ctx PROJ context, or NULL
+ * @return Endpoint URL. The returned pointer would be invalidated
+ * by a later call to proj_context_set_url_endpoint()
+ * @since 7.1
+ */
 const char *proj_context_get_url_endpoint(PJ_CONTEXT *ctx) {
     if (ctx == nullptr) {
         ctx = pj_get_default_ctx();
@@ -1236,11 +1262,13 @@ static std::string pj_get_relative_share_proj_internal_no_check() {
 #if defined(_WIN32) || defined(HAVE_LIBDL)
 #ifdef _WIN32
     HMODULE hm = NULL;
+#if !UWP
     if (GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS |
                               GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
                           (LPCSTR)&pj_get_relative_share_proj, &hm) == 0) {
         return std::string();
     }
+#endif // UWP
 
     DWORD path_size = 1024;
 
@@ -1267,7 +1295,7 @@ static std::string pj_get_relative_share_proj_internal_no_check() {
     constexpr char dir_sep = '\\';
 #else
     Dl_info info;
-    if (!dladdr((const void *)pj_get_relative_share_proj, &info)) {
+    if (!dladdr((void *)pj_get_relative_share_proj, &info)) {
         return std::string();
     }
     std::string out(info.dli_fname);
@@ -1392,6 +1420,22 @@ static void *pj_open_lib_internal(
         if (out_full_filename != nullptr && out_full_filename_size > 0)
             out_full_filename[0] = '\0';
 
+        auto open_lib_from_paths = [&ctx, open_file, &name, &fname, &sysname,
+                                    &mode](const std::string &projLibPaths) {
+            void *lib_fid = nullptr;
+            auto paths = NS_PROJ::internal::split(projLibPaths, dirSeparator);
+            for (const auto &path : paths) {
+                fname = NS_PROJ::internal::stripQuotes(path);
+                fname += DIR_CHAR;
+                fname += name;
+                sysname = fname.c_str();
+                lib_fid = open_file(ctx, sysname, mode);
+                if (lib_fid)
+                    break;
+            }
+            return lib_fid;
+        };
+
         /* check if ~/name */
         if (is_tilde_slash(name))
             if ((sysname = getenv("HOME")) != nullptr) {
@@ -1459,16 +1503,7 @@ static void *pj_open_lib_internal(
         else if (!gbPROJ_LIB_ENV_VAR_TRIED_LAST &&
                  !(projLib = NS_PROJ::FileManager::getProjLibEnvVar(ctx))
                       .empty()) {
-            auto paths = NS_PROJ::internal::split(projLib, dirSeparator);
-            for (const auto &path : paths) {
-                fname = path;
-                fname += DIR_CHAR;
-                fname += name;
-                sysname = fname.c_str();
-                fid = open_file(ctx, sysname, mode);
-                if (fid)
-                    break;
-            }
+            fid = open_lib_from_paths(projLib);
         }
 
         else if ((sysname = get_path_from_relative_share_proj(
@@ -1491,16 +1526,7 @@ static void *pj_open_lib_internal(
         else if (gbPROJ_LIB_ENV_VAR_TRIED_LAST &&
                  !(projLib = NS_PROJ::FileManager::getProjLibEnvVar(ctx))
                       .empty()) {
-            auto paths = NS_PROJ::internal::split(projLib, dirSeparator);
-            for (const auto &path : paths) {
-                fname = path;
-                fname += DIR_CHAR;
-                fname += name;
-                sysname = fname.c_str();
-                fid = open_file(ctx, sysname, mode);
-                if (fid)
-                    break;
-            }
+            fid = open_lib_from_paths(projLib);
         }
 
         else {
@@ -1763,11 +1789,37 @@ void pj_load_ini(PJ_CONTEXT *ctx) {
     if (ctx->iniFileLoaded)
         return;
 
+    // Start reading environment variables that have priority over the
+    // .ini file
+    const char *proj_network = getenv("PROJ_NETWORK");
+    if (proj_network && proj_network[0] != '\0') {
+        ctx->networking.enabled = ci_equal(proj_network, "ON") ||
+                                  ci_equal(proj_network, "YES") ||
+                                  ci_equal(proj_network, "TRUE");
+    } else {
+        proj_network = nullptr;
+    }
+
     const char *endpoint_from_env = getenv("PROJ_NETWORK_ENDPOINT");
     if (endpoint_from_env && endpoint_from_env[0] != '\0') {
         ctx->endpoint = endpoint_from_env;
     }
 
+    // Custom path to SSL certificates.
+    const char *ca_bundle_path = getenv("PROJ_CURL_CA_BUNDLE");
+    if (ca_bundle_path == nullptr) {
+        // Name of environment variable used by the curl binary
+        ca_bundle_path = getenv("CURL_CA_BUNDLE");
+    }
+    if (ca_bundle_path == nullptr) {
+        // Name of environment variable used by the curl binary (tested
+        // after CURL_CA_BUNDLE
+        ca_bundle_path = getenv("SSL_CERT_FILE");
+    }
+    if (ca_bundle_path != nullptr) {
+        ctx->ca_bundle_path = ca_bundle_path;
+    }
+
     ctx->iniFileLoaded = true;
     auto file = std::unique_ptr<NS_PROJ::File>(
         reinterpret_cast<NS_PROJ::File *>(pj_open_lib_internal(
@@ -1799,13 +1851,10 @@ void pj_load_ini(PJ_CONTEXT *ctx) {
                 trim(content.substr(equal + 1, eol - (equal + 1)));
             if (ctx->endpoint.empty() && key == "cdn_endpoint") {
                 ctx->endpoint = value;
-            } else if (key == "network") {
-                const char *enabled = getenv("PROJ_NETWORK");
-                if (enabled == nullptr || enabled[0] == '\0') {
-                    ctx->networking.enabled = ci_equal(value, "ON") ||
-                                              ci_equal(value, "YES") ||
-                                              ci_equal(value, "TRUE");
-                }
+            } else if (proj_network == nullptr && key == "network") {
+                ctx->networking.enabled = ci_equal(value, "ON") ||
+                                          ci_equal(value, "YES") ||
+                                          ci_equal(value, "TRUE");
             } else if (key == "cache_enabled") {
                 ctx->gridChunkCache.enabled = ci_equal(value, "ON") ||
                                               ci_equal(value, "YES") ||
@@ -1828,6 +1877,8 @@ void pj_load_ini(PJ_CONTEXT *ctx) {
                         ctx, PJ_LOG_ERROR,
                         "pj_load_ini(): Invalid value for tmerc_default_algo");
                 }
+            } else if (ca_bundle_path == nullptr && key == "ca_bundle_path") {
+                ctx->ca_bundle_path = value;
             }
         }
 
@@ -1931,6 +1982,7 @@ void proj_context_set_ca_bundle_path(PJ_CONTEXT *ctx, const char *path) {
         ctx = pj_get_default_ctx();
     if (!ctx)
         return;
+    pj_load_ini(ctx);
     try {
         ctx->set_ca_bundle_path(path != nullptr ? path : "");
     } catch (const std::exception &) {
